#ifdef HAVE_CONFIG_H
#include "fix_configs.h"
#endif
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include "message.h"

// if defined to 1:
//   ignore verbosity level when m_ferror is called ( display the message even if V_SILENT is the verbosity level )
// else if defined to 0:
//   respect verbosity level when m_ferror is called ( don't display message if V_SILENT is the verbosity level )
#define IGNORE_VERBOSITY_ON_FATAL_ERROR	1

static V_LEVEL verbosity = V_WARNINGS; // default
static FILE *logto = NULL, *logerrto = NULL;
static void sanity(void)
{
	if((logto == NULL) || (logerrto == NULL))
	{
		if(logto == NULL) logto = stdout;
		if(logerrto == NULL) logerrto = stderr;
	}
}
void m_info(const char *fmt, ...)
{
	sanity();
    if(verbosity >= V_VERBOSE)
    {
        va_list arg_list;
        va_start(arg_list, fmt);
        vfprintf(logto, fmt, arg_list);
        va_end(arg_list);
    }
}

void m_warning(const char *fmt, ...)
{
	sanity();
    if(verbosity >= V_WARNINGS)
    {
        va_list arg_list;
        va_start(arg_list, fmt);
        vfprintf(logto, fmt, arg_list);
        va_end(arg_list);
    }
}

void m_error(const char *fmt, ...)
{
	sanity();
    if(verbosity >= V_ERRORS)
    {
        va_list arg_list;
        va_start(arg_list, fmt);
        vfprintf(logerrto, fmt, arg_list);
        va_end(arg_list);
    }
}

void m_ferror(int status, const char *fmt, ...)
{
	sanity();
	#if ! IGNORE_VERBOSITY_ON_FATAL_ERROR
	if(verbosity >= V_ERRORS)
	{
	#endif
	va_list arg_list;
	va_start(arg_list, fmt);
	vfprintf(logerrto, fmt, arg_list);
	va_end(arg_list);
	exit(status);
	#if ! IGNORE_VERBOSITY_ON_FATAL_ERROR
	}
	#endif
}

void m_set_fp(FILE *logfilep, FILE *errlogfilep)
{
	logto = logfilep;
	logerrto = errlogfilep;
}

void m_set_verbosity(V_LEVEL v)
{
	verbosity = v;
}

V_LEVEL m_get_verbosity(void)
{
	return verbosity;
}
