/****************************************************************************
*                      RECAFOH (pronounced REC-ah-foe)                      *
*                                                                           *
*           Written By Benjamin Lee Wiebe <itsme@benjiwiebe.com>            *
*                                                                           *
*          Remotely Execute a Command in an Alias File On the Host          *
****************************************************************************/
#ifdef HAVE_CONFIG_H
#include "fix_configs.h"
#endif
#include <arpa/inet.h>
#include <getopt.h>
#include <errno.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <assert.h>
#include "message.h"

char *prg_invocname = NULL;
void error(const char *);
void print_help(char*);
void mem_free_all(void);
char *command = NULL;
char *hostname = NULL;
int port = 2012; //default port
int main(int argc, char *argv[])
{
    #ifndef RELEASE_BUILD
    char *cptmp = NULL;
    #endif
    struct hostent *hp;
    char buffer[BUFSIZ];
    int itmp = 0;

    static struct option long_options[] =
    {
        {"port", required_argument, 0, 'p'},
        {"command", required_argument, 0, 'c'},
        {"verbose", no_argument, 0, 'v'},
        {"warnings", no_argument, 0, 'w'},
        {"errors", no_argument, 0, 'e'},
        {"silent", no_argument, 0, 's'},
        {"help", no_argument, 0, 'h'}
    };

    prg_invocname = (char*)malloc(strlen(argv[0]));
    strcpy(prg_invocname, argv[0]);

    atexit(mem_free_all);
    int option = 0;
    command = malloc(BUFSIZ+1);

    while((option = getopt_long_only(argc, argv, "p:c:vwesh", long_options, 0)) != (-1))
    {
        switch(option)
        {
        case 'p':
            assert(optarg != (char*)0);
            port = atoi(optarg);
            break;
        case 'c':
            assert(optarg != (char*)0);
            itmp = strlen(optarg);
            if(command != NULL)
		free(command);
            command = (char*)malloc(itmp);
            memset(command, 0, itmp);
            strcpy(command, optarg);
            break;
        case 'v':
            m_set_verbosity(V_VERBOSE);
            break;
        case 'w':
            m_set_verbosity(V_WARNINGS);
            break;
        case 'e':
            m_set_verbosity(V_ERRORS);
            break;
        case 's':
            m_set_verbosity(V_SILENT);
            break;
        case 'h':
            print_help(argv[0]);
            exit(EXIT_SUCCESS);
        default:
            //m_error("%s: Unknown option.\r\n", prg_invocname);
            exit(EXIT_FAILURE);
        }
    }

    // initialize the message code...
    m_set_fp(stdout, stderr);

    if(optind == argc)
    {
        m_error("No host name supplied.\r\n");
        exit(EXIT_FAILURE);
    }
    else if((optind + 1) == argc)
    {
        hostname = (char*) malloc(strlen(argv[optind]));
        hostname[0] = '\0';
        strcpy(hostname, argv[optind]);
        hp = gethostbyname(argv[optind]);
        if(hp == NULL)
        {
            fprintf(stderr, "Error - gethostbyname() failed: ");
            switch(h_errno)
            {
            case HOST_NOT_FOUND:
                fprintf(stderr, "The specified host is unknown.\r\n");
                break;
            case NO_DATA:
                fprintf(stderr, "The requested name is valid but does not have an IP address.\r\n");
                break;
            case NO_RECOVERY:
                fprintf(stderr, "A nonrecoverable name server error occurred.\r\n");
                break;
            case TRY_AGAIN:
                fprintf(stderr, "A temporary error occurred on an authoritative name server.  Try again later.\r\n");
                break;
            }
            exit(EXIT_FAILURE);
        }
        #ifndef RELEASE_BUILD
                if(m_get_verbosity() == V_VERBOSE)
                {
                    itmp = strlen(hp->h_addr_list[0]);
                    cptmp = (char*) malloc(itmp + 1);
                    if(cptmp == NULL)
                        m_ferror(EXIT_FAILURE, "%s: Failed to allocate %d bytes of memory: %s.\r\n", prg_invocname, itmp+1, strerror(errno));
                    memset(cptmp, 0, itmp-1);
                    errno = 0;
                    if(inet_ntop(AF_INET, hp->h_addr_list[0], cptmp, itmp) == NULL)
                    {
                        m_warning("%s: Could not map binary IP address to textual IP address: %s.\r\n", prg_invocname, strerror(errno));
                    }
                    m_info("%s: Host to connect to is '%s', IP '%s'.\r\n", prg_invocname, argv[optind], cptmp);
                    free(cptmp);
                }
        #endif
    }
    else
    {
        print_help(argv[0]);
        exit(EXIT_FAILURE);
    }
/** START***CLIENT*** */
    int sock, n;
    struct sockaddr_in server;
    struct sockaddr_in from;
    memset(&server, 0, sizeof(struct sockaddr_in));
    memset(&from, 0, sizeof(struct sockaddr_in));

    sock = socket(AF_INET, SOCK_STREAM, 0);
//    sock = socket(AF_INET, SOCK_DGRAM, 0);
    if(sock < 0)
        m_error("Opening socket failed: %s.\r\n", strerror(errno));
    else
        m_info("Opened socket.\r\n");

    server.sin_family = AF_INET;
    bcopy((char*)hp->h_addr,
          (char*)&server.sin_addr,
          hp->h_length);
    server.sin_port = htons(port);

    if((isatty(fileno(stdin)))&&(strlen(command)==0)) /* IF command is not specified on command line AND stdin is a tty (not piped) THEN print prompt */
    {
        printf("Command: ");
    }

    memset(buffer, 0, BUFSIZ);
    if(strlen(command)) /* if command is specified on command line... */
    {
        strcpy(buffer, command);
    }
    else /* else read it in from standard input... */
    {
        fgets(buffer, BUFSIZ-1, stdin);
    }

    if(connect(sock, (struct sockaddr*)&server, sizeof(server)) < 0)
    {
        m_ferror(EXIT_FAILURE, "Failed to connect to port %d: %s.\r\n", port, strerror(errno));
    }
//    n = sendto(sock, buffer, strlen(buffer), 0, (const struct sockaddr*)&server, length);
    n = write(sock, buffer, strlen(buffer));
    if(n < 0)
        m_ferror(EXIT_FAILURE, "write() failed: %s.\r\n", strerror(errno));

//    n = recvfrom(sock, buffer, BUFSIZ, 0, (struct sockaddr*)&from, &length);
    n = read(sock, buffer, BUFSIZ);
    if(n < 0)
        m_ferror(EXIT_FAILURE, "read() failed: %s.\r\n", strerror(errno));

    printf("Message from %s: %s\r\n", hostname, buffer); // here, we assume that buffer contains no null chars (other then null terminator)
                                   //   do *not* let the server ever send NULL chars in the reply
    close(sock);
/** END***SERVER*** */
    exit(EXIT_SUCCESS);
}

void error(const char *msg)
{
    perror(msg);
    exit(0);
}

void print_help(char *prg)
{
	printf("Usage: %s [options] hostname\r\n", prg);
	printf("  -p, --port\t\tThe port number to use\r\n");
	printf("  -c, --command\t\tThe command to run\r\n");
	printf("  -v, --verbose\t\tPrint everything\r\n");
	printf("  -w, --warnings\tPrint warnings and errors (default)\r\n");
	printf("  -e, --errors\t\tPrint the errors and nothing else\r\n");
	printf("  -s, --silent\t\tPrint nothing at all\r\n");
	printf("  -h, --help\t\tPrint this help message and exit\r\n");
	exit(EXIT_SUCCESS);
}

void mem_free_all(void)
{
    if(prg_invocname != NULL)
        free(prg_invocname);
    if(command != NULL)
	free(command);
    if(hostname != NULL)
        free(hostname);
}
