#include "shared.h"
#ifndef USE_SYSTEMD
#error "This file should not be compiled when --disable-systemd was passed to configure"
#endif
void install_service(char *prg, const char *service_filename)
{
    char *prg_filename = NULL;
    m_info("%s: Creating file '%s'...\r\n", prg, service_filename);
    if(geteuid())
    {
        printf("You must be root to install the systemd recafohd.service file.\r\n");
        exit(EXIT_FAILURE);
    }
    FILE *service;
    service = fopen(service_filename, "r");
    if(service)
    {
        m_warning("%s: Service file '%s' already exists. Deleting...\r\n", prg, service_filename);
        if(xunlink(service_filename) < 0)
        {
            m_error("%s: Could not delete file %s: %s\r\n.", prg, service_filename, strerror(errno));
            exit(EXIT_FAILURE);
        }
        fclose(service);
    }
    service = fopen(service_filename, "w+");
    if(service == NULL)
    {
        m_ferror(EXIT_FAILURE, "%s: Could not open %s for writing: %s.\r\n", prg, service_filename, strerror(errno));
    }
    fprintf(service, "[Unit]\r\n");
    fprintf(service, "Description=RECAFOH daemon\r\n");
    fprintf(service, "After=network.target\r\n");
    fprintf(service, "\r\n");
    fprintf(service, "[Service]\r\n");
    fprintf(service, "Type=forking\r\n");
    prg_filename = get_prg_filename(getpid());
    assert(prg_filename != NULL);
    fprintf(service, "ExecStart=%s --forking\r\n", prg_filename);
    free(prg_filename);
    fprintf(service, "\r\n");
    fprintf(service, "[Install]\r\n");
    if(fprintf(service, "WantedBy=multi-user.target\r\n") < 0)
        m_ferror(EXIT_FAILURE, "%s: Failed to write to file '%s': %s.\r\n", prg, service_filename, strerror(errno));
    m_info("%s: Successfully created file '%s'.\r\n", prg, service_filename);
    exit(EXIT_SUCCESS);
}

void remove_service(char *prg, const char *service_filename)
{
    m_info("%s: Deleting file '%s'...\r\n", service_filename);
    FILE *service;
    service = fopen(service_filename, "r");
    if(service)
    {
        fclose(service);
        if(xunlink(service_filename) < 0)
        {
            m_error("%s: Could not delete file %s: %s.\r\n", prg, service_filename, strerror(errno));
            exit(EXIT_FAILURE);
        }
        else
        {
            m_info("%s: Removed file successfully.\r\n");
            return;
        }
    }
    else if(errno == ENOENT)
    {
        m_warning("%s: File '%s' does not exist.\r\n", prg, service_filename);
        return;
    }
    else
        m_ferror(EXIT_FAILURE, "%s: Could not delete file '%s': %s.\r\n", prg, service_filename, strerror(errno));
}
